@@ -45,6 +45,11 @@ class VM(object):
         else:
             _, self.iso_path = tempfile.mkstemp(suffix='.iso', dir=temp_dir)
 
+    def vminfo(self):
+        """Returns a dictionary with all available information for the
+        Virtual Machine."""
+        raise
+
     def create_vm(self):
         """Create a new Virtual Machine."""
         raise
@@ -65,6 +70,14 @@ class VM(object):
         """Create a harddisk."""
         raise
 
+    def immutable_hd(self):
+        """Make a harddisk immutable or normal."""
+        raise
+
+    def remove_hd(self):
+        """Remove a harddisk."""
+        raise
+
     def cpus(self, count):
         """Set the number of CPUs to assign to this Virtual Machine."""
         raise
