@@ -3,17 +3,19 @@
 # See the file 'docs/LICENSE' for copying permission.
 
 import os
-import sys
-import time
+import platform
+import random
 import socket
 import string
-import random
-import platform
+import sys
+import time
 import subprocess
-import ConfigParser
-from StringIO import StringIO
-from zipfile import ZipFile
+from ConfigParser import RawConfigParser
 from SimpleXMLRPCServer import SimpleXMLRPCServer
+from StringIO import StringIO
+from _winreg import CreateKeyEx, DeleteValue
+from _winreg import HKEY_LOCAL_MACHINE, KEY_ALL_ACCESS
+from zipfile import ZipFile
 
 BIND_IP = "0.0.0.0"
 BIND_PORT = 8000
@@ -94,10 +96,10 @@ class Agent:
         file_path = os.path.join(root, name)
 
         try:
-            with open(file_path, "wb") as sample:
-                sample.write(data)
+            with open(file_path, "wb") as malware:
+                malware.write(data)
         except IOError as e:
-            ERROR_MESSAGE = "Unable to write sample to disk: {0}".format(e)
+            ERROR_MESSAGE = "Unable to write malware to disk: {0}".format(e)
             return False
 
         return True
@@ -109,10 +111,10 @@ class Agent:
         """
         global ERROR_MESSAGE
 
-        if not isinstance(options, dict):
+        if type(options) != dict:
             return False
 
-        config = ConfigParser.RawConfigParser()
+        config = RawConfigParser()
         config.add_section("analysis")
 
         try:
@@ -136,14 +138,6 @@ class Agent:
 
         return True
 
-    def add_hashes(self, hashes):
-        hashes_path = os.path.join(ANALYZER_FOLDER, "hashes.bin")
-
-        with open(hashes_path, "wb") as f:
-            f.write(hashes.data)
-
-        return True
-
     def add_analyzer(self, data):
         """Add analyzer.
         @param data: analyzer data.
@@ -211,16 +205,56 @@ class Agent:
         return True
 
 if __name__ == "__main__":
+    vmmode = sys.argv[1]
+
+    # Retrieve the IP and Port of the host machine.
+    host_ip = sys.argv[2]
+    host_port = int(sys.argv[3])
+
+    # Attempt to connect to the host machine.
+    s = None
+    while not s:
+        try:
+            s = socket.create_connection((host_ip, host_port), 1)
+        except socket.error:
+            continue
+
+    # Connect to the host machine. In case this is a bird, also receive the
+    # new IP address, mask, and gateway.
+    if vmmode == 'bird':
+        # Retrieve the static IP address that we're supposed to use.
+        ip_address, ip_mask, ip_gateway = s.recv(256).split()
+
+        s.close()
+
+        args = [
+            "netsh", "interface", "ip", "set", "address",
+            "name=Local Area Connection", "static",
+            ip_address, ip_mask, ip_gateway, "1",
+        ]
+        subprocess.Popen(args).wait()
+    else:
+        s.close()
+
     try:
+        # Remove the entry in Run from the registry.
+        h = CreateKeyEx(HKEY_LOCAL_MACHINE,
+                        "Software\\Microsoft\\Windows\\CurrentVersion\\Run",
+                        0, KEY_ALL_ACCESS)
+        DeleteValue(h, "Agent")
+        h.Close()
+
         if not BIND_IP:
             BIND_IP = socket.gethostbyname(socket.gethostname())
 
+        # Remove this file.
+        os.unlink(os.path.abspath(__file__))
+
         print("[+] Starting agent on %s:%s ..." % (BIND_IP, BIND_PORT))
 
         # Disable DNS lookup, by Scott D.
         def FakeGetFQDN(name=""):
             return name
-
         socket.getfqdn = FakeGetFQDN
 
         server = SimpleXMLRPCServer((BIND_IP, BIND_PORT), allow_none=True)
