@@ -18,6 +18,7 @@ except ImportError:
 
 from vmcloak.abstract import VM
 from vmcloak.data.config import VBOX_CONFIG
+from vmcloak.exceptions import CommandError
 from vmcloak.rand import random_mac
 
 log = logging.getLogger(__name__)
@@ -39,16 +40,30 @@ class VirtualBox(VM):
             if v is None or v is True:
                 cmd += ['--' + k]
             else:
-                cmd += ['--' + k, str(v)]
+                cmd += ['--' + k.rstrip('_'), str(v)]
 
         try:
             ret = subprocess.check_output(cmd)
         except Exception as e:
             log.error('[-] Error running command: %s', e)
-            exit(1)
+            raise CommandError
 
         return ret.strip()
 
+    def vminfo(self):
+        ret = {}
+        lines = self._call('showvminfo', self.name, machinereadable=True)
+        for line in lines.split('\n'):
+            key, value = line.split('=', 1)
+
+            if value.startswith('"') and value.endswith('"'):
+                value = value[1:-1]
+            elif value.isdigit():
+                value = int(value)
+
+            ret[key] = value
+        return ret
+
     def api_status(self):
         if not os.path.isfile(self.vboxmanage):
             log.error('VBoxManage not found!')
@@ -73,24 +88,33 @@ class VirtualBox(VM):
         return self._call('modifyvm', self.name, ostype=operating_systems[os])
 
     def create_hd(self, fsize):
-        ctlname = 'IDE Controller'
         self._call('createhd', filename=self.hdd_path, size=fsize)
-        self._call('storagectl', self.name, name=ctlname, add='ide')
-        self._call('storageattach', self.name, storagectl=ctlname,
-                   type='hdd', device=0, port=0, medium=self.hdd_path)
+        self._call('storagectl', self.name, name='IDE', add='ide')
+        self._call('storageattach', self.name, storagectl='IDE',
+                   type_='hdd', device=0, port=0, medium=self.hdd_path)
+
+    def attach_hd(self, path):
+        self._call('storagectl', self.name, name='IDE', add='ide')
+        self._call('storageattach', self.name, storagectl='IDE',
+                   type_='hdd', device=0, port=0, medium=path)
+
+    def immutable_hd(self):
+        self._call('modifyhd', self.hdd_path, type_='immutable', compact=True)
+
+    def remove_hd(self):
+        self._call('storagectl', self.name, portcount=0,
+                   name='IDE', remove=True)
 
     def cpus(self, count):
         self._call('modifyvm', self.name, cpus=count, ioapic='on')
 
     def attach_iso(self, iso):
-        ctlname = 'IDE Controller'
-        self._call('storageattach', self.name, storagectl=ctlname,
-                   type='dvddrive', port=1, device=0, medium=iso)
+        self._call('storageattach', self.name, storagectl='IDE',
+                   type_='dvddrive', port=1, device=0, medium=iso)
 
     def detach_iso(self):
-        ctlname = 'IDE Controller'
-        self._call('storageattach', self.name, storagectl=ctlname,
-                   type='dvddrive', port=1, device=0, medium='emptydrive')
+        self._call('storageattach', self.name, storagectl='IDE',
+                   type_='dvddrive', port=1, device=0, medium='emptydrive')
 
     def set_field(self, key, value):
         return self._call('setextradata', self.name, key, value)
@@ -158,7 +182,7 @@ class VirtualBox(VM):
 
     def start_vm(self, visible=False):
         return self._call('startvm', self.name,
-                          type='gui' if visible else 'headless')
+                          type_='gui' if visible else 'headless')
 
     def snapshot(self, label, description=''):
         return self._call('snapshot', self.name, 'take', label,
@@ -328,3 +352,44 @@ class VBoxRPC(VM):
     def vrde(self, vrde):
         vrde = 'on' if vrde else 'off'
         return self._query('modifyvm', self.name, vrde=vrde)
+
+
+def initialize_vm(m, s, clone=False):
+    log.debug('Creating VM %r.', s.vmname)
+    log.debug(m.create_vm())
+
+    m.ramsize(s.ramsize)
+    m.os_type(os='xp', sp=3)
+
+    if not clone:
+        log.debug('Creating Harddisk.')
+        m.create_hd(s.hdsize)
+
+        log.debug('Temporarily attaching DVD-Rom unit for the ISO installer.')
+        m.attach_iso(m.iso_path)
+
+    log.debug('Randomizing Hardware.')
+    m.init_vm(profile=s.hwconfig_profile)
+
+    log.debug('Setting CPU count to %d', s.cpu_count)
+    m.cpus(s.cpu_count)
+
+    log.debug('Checking VirtualBox hostonly network.')
+    if not m.hostonly(macaddr=s.hostonly_macaddr):
+        exit(1)
+
+    if s.nat:
+        m.nat()
+
+    if s.bridged:
+        m.bridged(s.bridged, macaddr=s.bridged_macaddr)
+
+    if s.hwvirt is not None:
+        if s.hwvirt:
+            log.debug('Enabling Hardware Virtualization.')
+        else:
+            log.debug('Disabling Hardware Virtualization.')
+        m.hwvirt(s.hwvirt)
+
+    if s.vrde:
+        m.vrde(True)
